
!!! Algorthmische Komplexität

!!!!!!!!!!!!!!!
!!! Programm zum Berechnen des Kreisumfang
!!! In Abhängigkeit von der Eingabe n
PROGRAM circle
	IMPLICIT NONE

	REAL           :: n, u
	REAL,PARAMETER :: pi = 3.14159265359  !1

	READ(*,*) n

	u = pi*2*n                            !1 (Berechnung mit Grundoperation)
    ! Bei MATMUL wäre das nicht 1 --> intrinsische Funktionen beachten
END PROGRAM circle

! Laufzeitkomplexität: 1+1 = 2 --> als O(1)
! --> konstante Laufzeitkomplexität
! Speicherkomplexität: drei REAL-Variablen: 3 --> O(1)
!!!!!!!!!!!!!!!!


!!!!!!!!!!!!!!!!!
!!! Programm berechnet x^n iterativ
! In Abhängigkeit von den Eingaben x UND n
FUNCTION expon_it(x, n)
  REAL    :: x, expon_it
  INTEGER :: i,n

  expon_it = 1                !1
  DO i = 1, n                 !n-mal
    expon_it = expon_it * x   !1
  END DO
END FUNCTION expon_it
! Laufzeit: 1+n*1 = n+1 --> O(n)
!   --> lineare Laufzeitkomplexität
! Speicher: 4 skalare Variablen: 4 --> O(1)
!!!!!!!!!!!!


!!!!!!!!!!!!!!!!
!!! Eingabe -- INTEGER n
INTEGER:: n, i, j, a=0

DO i = 1, n         !n-mal
  DO j = 1, n-i    !(n-i)-mal
    a = a+1         !1
  END DO
END DO
!!!!!!!!!!!!!!!!
! Laufzeit: n*(n-i)*1
! (n-i) < n --> Abschätzen: n*n*1 = n^2
! --> Laufzeitkomplexität O(n^2) (quadratisch)

!!!!!!!!!!!!!!!!!
!!! Eingabe -- INTEGER n
INTEGER:: n, i, j, a=0
INTEGER,DIMENSION(:,:),ALLOCATABLE:: matrix

IF(n == 17) THEN      !1 (Vergleich!)
  a = 0               !1
ELSE
  ALLOCATE(matrix(n,n))   ! Speicherplatzbedarf: n*n = n^2
  DO i = 1, n         !n-mal
    DO j = 1, n       !n-mal
      a = a+1         !1
      matrix(i,j) = a !1
    END DO
  END DO
END IF

!!! Laufzeitkomplexität für n == 17 ist O(1)
!!!                         sonst      O(n^2)
!!! Also hier Unterscheidung in worst und best case
!!! best case:  T(n) = O(1)   und   S(n) = O(1)
!!! worst case: T(n) = O(n^2) und   S(n) = O(n^2) = average case


!!! Rekursive Funktionen
RECURSIVE FUNCTION s(n) RESULT (res)
  INTEGER:: n, res

  IF(n==0.AND.n==1) THEN
    res = 1
  ELSE
    res = s(n/2) + s(n-1) + s(n-2)
  END IF

END FUNCTION s
!!! Hier spielt Rekursionstiefe und Anzahl der rekursiven Aufrufe eine Rolle
!!! Laufzeit:
!!! maximale Rekursionstiefe: max(n/2, n-1, n-2) = n-1 <= n
!!! Anzahl der rekursiven Aufrufe pro rekursiven Aufruf: 3
!!! Insgesamt: Laufzeitkomplexität T(n) = O(3^n)
