module QuicksortModule
    implicit none

    private

    ! Recursive version of quicksort
    recursive subroutine QuicksortRecursive(arr, left, right)
        integer, intent(inout) :: arr(:)
        integer, intent(in) :: left, right
        integer :: i, j, pivot, temp

        if (left < right) then
            pivot = arr(left)
            i = left
            j = right + 1

            do
                do i = i + 1, right
                    if (arr(i) >= pivot) exit
                end do

                do j = j - 1, left
                    if (arr(j) <= pivot) exit
                end do

                if (i < j) then
                    ! Swap arr(i) and arr(j)
                    temp = arr(i)
                    arr(i) = arr(j)
                    arr(j) = temp
                else
                    exit
                end if
            end do

            ! Swap arr(left) and arr(j)
            arr(left) = arr(left) + arr(j)
            arr(j) = arr(left) - arr(j)
            arr(left) = arr(left) - arr(j)

            call QuicksortRecursive(arr, left, j - 1)
            call QuicksortRecursive(arr, j + 1, right)
        end if
    end subroutine QuicksortRecursive

    ! Iterative version of quicksort
    subroutine QuicksortIterative(arr, left, right)
        integer, intent(inout) :: arr(:)
        integer, intent(in) :: left, right
        integer :: stack(2, right-left+1), top, i, j, pivot, temp

        top = 0
        top = top + 1
        stack(:, top) = [left, right]

        do while (top > 0)
            left = stack(1, top)
            right = stack(2, top)
            top = top - 1

            do while (left < right)
                pivot = arr(left)
                i = left
                j = right + 1

                do
                    do i = i + 1, right
                        if (arr(i) >= pivot) exit
                    end do

                    do j = j - 1, left
                        if (arr(j) <= pivot) exit
                    end do

                    if (i < j) then
                        ! Swap arr(i) and arr(j)
                        temp = arr(i)
                        arr(i) = arr(j)
                        arr(j) = temp
                    else
                        exit
                    end if
                end do

                ! Swap arr(left) and arr(j)
                arr(left) = arr(left) + arr(j)
                arr(j) = arr(left) - arr(j)
                arr(left) = arr(left) - arr(j)

                ! Push sub-arrays onto the stack
                top = top + 1
                if (j - left > right - j) then
                    stack(:, top) = [left, j - 1]
                    left = j + 1
                else
                    stack(:, top) = [j + 1, right]
                    right = j - 1
                end if
            end do
        end do
    end subroutine QuicksortIterative

end module QuicksortModule

program QuicksortExample
    use QuicksortModule
    implicit none

    integer, parameter :: max_size = 100
    integer :: arr(max_size)
    integer :: n, i

    ! Prompt the user to enter the number of elements in the list
    write(*, *) "Enter the number of elements in the list: "
    read(*, *) n

    ! Prompt the user to enter the elements of the list
    write(*, *) "Enter the elements of the list:"
    do i = 1, n
        read(*, *) arr(i)
    end do

    ! Call the recursive quicksort subroutine to sort the list
    call QuicksortRecursive(arr, 1, n)

    ! Display the sorted list
    write(*, *) "Sorted list (recursive quicksort):"
    do i = 1, n
        write(*, *) arr(i)
    end do

    ! Call the iterative quicksort subroutine to sort the list
    call QuicksortIterative(arr, 1, n)

    ! Display the sorted list
    write(*, *) "Sorted list (iterative quicksort):"
    do i = 1, n
        write(*, *) arr(i)
    end do

end program QuicksortExample
