MODULE  STACKMOD

    IMPLICIT NONE
  
    PRIVATE
   ! export list for module STACKMOD:                                 !
    PUBLIC :: STACK, INIT, EMPTY, PUSH, POP, WRITE, DELETE,disc,top     
    type disc
        integer::num,color,diam
    end type disc
    TYPE  STACK
      integer::top_pos
      TYPE(STACKELEM), POINTER :: TOP
    END TYPE  STACK
   
    TYPE, PRIVATE :: STACKELEM      
      type(disc)              :: DATA
      TYPE(STACKELEM), POINTER :: SUCC
    END TYPE  STACKELEM
  
   CONTAINS
  
   ! initializes stack to a defined state (empty) !
    SUBROUTINE  INIT (S)
      TYPE(STACK), INTENT(OUT) :: S
      s%top_pos=0
      NULLIFY (S%TOP)
    END SUBROUTINE  INIT
  
   ! tests if stack is currently empty !
    FUNCTION  EMPTY (S)
      TYPE(STACK), INTENT(IN) :: S
      LOGICAL                 :: EMPTY 
      EMPTY =  .NOT. ASSOCIATED(S%TOP)
    END FUNCTION  EMPTY 
  
   ! adds new top element to stack !
    SUBROUTINE  PUSH (S, ELEM)
      TYPE(STACK), INTENT(INOUT) :: S
      type(disc), INTENT(IN)  :: ELEM
      TYPE(STACKELEM), POINTER   :: PTR
      INTEGER                    :: err_code
      ALLOCATE (PTR, STAT = err_code)
      IF ( err_code == 0 ) THEN
        PTR%DATA = ELEM
        PTR%SUCC => S%TOP
        S%TOP => PTR
        S%Top_pos=s%top_pos+1
      ELSE
        WRITE(*,*) ' Allocation failed, probably out of memory!'
      END IF
    END SUBROUTINE  PUSH 
  
   ! removes top element from stack and returns its CONTENT !            ! 4 !
   ! via the second argument ELEM, if PRESENT               !
    SUBROUTINE  POP (S, ELEM)
      TYPE(STACK), INTENT(INOUT)           :: S
      type(disc), INTENT(OUT), OPTIONAL :: ELEM
      TYPE(STACKELEM), POINTER             :: PTR
      INTEGER                              :: err_code
      IF ( .NOT. EMPTY(S) ) THEN
        PTR => S%TOP
        IF ( PRESENT(ELEM) )  ELEM = PTR%DATA
        S%TOP => PTR%SUCC
        DEALLOCATE (PTR, STAT = err_code)

        IF ( err_code /= 0 )then                                     
         WRITE(*,*) ' Deallocation failed, inconsistent memory!'
        else
            S%top_pos=S%top_pos-1
        end if
      ELSE
        WRITE(*,*) ' Cannot pop from empty stack!'
      END IF
    END SUBROUTINE  POP 

   ! returns CONTENT of top element !
    FUNCTION  TOP (S)
        TYPE(STACK), INTENT(IN) :: S
        type(disc)           :: TOP
        IF ( .NOT. EMPTY(S) ) THEN
          TOP = S%TOP%DATA
        ELSE                !!! Output in a function can be a PROBLEM !!!
          WRITE(*,*) ' Cannot get top of empty stack!'
        END IF
      END FUNCTION  TOP
   ! writes complete stack from top to bottom       !                    ! 5 !
   ! (low level - should be inside module STACKMOD) !
    SUBROUTINE  WRITE (S)
      TYPE(STACK), INTENT(IN)  :: S
      TYPE(STACKELEM), POINTER :: PTR
      PTR => S%TOP
      DO WHILE ( ASSOCIATED(PTR) )
        write(*,*)PTR%DATA%num
        PTR => PTR%SUCC
      END DO
      WRITE(*,*)
    END SUBROUTINE  WRITE
  
    SUBROUTINE  DELETE (S)
      TYPE(STACK), INTENT(INOUT) :: S
      TYPE(STACKELEM), POINTER   :: PTR
      INTEGER                    :: err_code
      DO WHILE ( ASSOCIATED(S%TOP) )
        PTR => S%TOP
        S%TOP => PTR%SUCC
        DEALLOCATE (PTR, STAT = err_code)
        IF ( err_code /= 0 ) THEN
          WRITE(*,*) ' Deallocation failed, inconsistent memory!'
          EXIT   ! >----------------------------------->>> EXIT !
        END IF
      END DO
    END SUBROUTINE  DELETE
  
  
  
  END MODULE  STACKMOD

PROGRAM TowersOfHanoi
    USE STACKMOD
    IMPLICIT NONE

    INTEGER, PARAMETER :: num_discs = 3

    TYPE(disc) :: discs(num_discs)
    INTEGER :: i

    ! Initialize the stacks
    TYPE(STACK) :: source_stack, auxiliary_stack, destination_stack
    CALL INIT(source_stack)
    CALL INIT(auxiliary_stack)
    CALL INIT(destination_stack)

    ! Create discs and push them onto the source stack
    DO i = num_discs, 1, -1
        discs(i)%num = i
        discs(i)%color = i
        discs(i)%diam = 2 * i
        CALL PUSH(source_stack, discs(i))
    END DO

    ! Call the recursive subroutine to solve the Towers of Hanoi puzzle
    WRITE(*, *) "Initial State:"
    WRITE(source_stack)
    WRITE(auxiliary_stack)
    WRITE(destination_stack)
    WRITE(*, *)
    CALL SolveTowersOfHanoi(num_discs, source_stack, auxiliary_stack, destination_stack)

    ! Clean up memory
    CALL DELETE(source_stack)
    CALL DELETE(auxiliary_stack)
    CALL DELETE(destination_stack)

END PROGRAM TowersOfHanoi

SUBROUTINE SolveTowersOfHanoi(n, source, auxiliary, destination)
    USE STACKMOD
    IMPLICIT NONE

    INTEGER, INTENT(IN) :: n
    TYPE(STACK), INTENT(INOUT) :: source, auxiliary, destination

    TYPE(disc) :: disc

    IF (n == 1) THEN
        ! Move the top disc from the source to the destination peg
        CALL POP(source, disc)
        CALL PUSH(destination, disc)
        WRITE(*, *) "Move disc ", disc%num, " from peg ", CHAR(source), " to peg ", CHAR(destination)
        WRITE(source)
        WRITE(auxiliary)
        WRITE(destination)
        WRITE(*, *)
    ELSE
        ! Recursive case:
        ! 1. Move (n-1) discs from source to auxiliary peg using destination peg as the auxiliary
        CALL SolveTowersOfHanoi(n - 1, source, destination, auxiliary)

        ! 2. Move the nth disc from source to destination peg
        CALL POP(source, disc)
        CALL PUSH(destination, disc)
        WRITE(*, *) "Move disc ", disc%num, " from peg ", CHAR(source), " to peg ", CHAR(destination)
        WRITE(source)
        WRITE(auxiliary)
        WRITE(destination)
        WRITE(*, *)

        ! 3. Move (n-1) discs from auxiliary peg to destination peg using source peg as the auxiliary
        CALL SolveTowersOfHanoi(n - 1, auxiliary, source, destination)
    END IF

END SUBROUTINE SolveTowersOfHanoi
